@@ -80,25 +80,28 @@
 fcount <- function(data, ..., wt = NULL, sort = FALSE, order = TRUE,
                    name = NULL, .by = NULL, .cols = NULL){
   group_vars <- group_vars(data)
-  group_info <- group_info(data, ..., .by = {{ .by }},
-                           .cols = .cols,
-                           ungroup = TRUE,
-                           rename = TRUE)
+  group_info <- tidy_group_info(data, ..., .by = {{ .by }},
+                                .cols = .cols,
+                                ungroup = TRUE,
+                                rename = TRUE)
   out <- group_info[["data"]]
   all_vars <- group_info[["all_groups"]]
   N <- df_nrow(out)
   # Weights
   if (!rlang::quo_is_null(enquo(wt))){
-    out <- mutate2(out, !!enquo(wt))
-    wt_var <- tidy_transform_names(data, !!enquo(wt))
+    out_info <- mutate_summary_grouped(out, !!enquo(wt))
+    wt_var <- out_info[["cols"]]
+    out <- out_info[["data"]]
   } else {
     wt_var <- character()
   }
   if (length(wt_var) > 0L){
     wtv <- out[[wt_var]]
   }
-  use_only_grouped_df_groups <- length(all_vars) == 0L ||
-    (order && length(group_vars) > 0L && length(group_vars) == length(all_vars))
+  use_only_grouped_df_groups <- !group_info[["groups_changed"]] && (
+    length(all_vars) == 0L ||
+      (order && length(group_vars) > 0L && length(group_vars) == length(all_vars))
+  )
   if (use_only_grouped_df_groups){
     g <- df_to_GRP(data, return.order = FALSE, order = order)
   } else {
@@ -151,20 +154,21 @@ fcount <- function(data, ..., wt = NULL, sort = FALSE, order = TRUE,
 fadd_count <- function(data, ..., wt = NULL, sort = FALSE, order = TRUE,
                        name = NULL, .by = NULL, .cols = NULL){
   group_vars <- group_vars(data)
-  group_info <- group_info(data, ..., .by = {{ .by }},
-                           .cols = .cols,
-                           ungroup = TRUE,
-                           rename = TRUE)
+  group_info <- tidy_group_info(data, ..., .by = {{ .by }},
+                                .cols = .cols,
+                                ungroup = TRUE,
+                                rename = TRUE)
   out <- group_info[["data"]]
   all_vars <- group_info[["all_groups"]]
   if (rlang::quo_is_null(enquo(wt))){
     wt_var <- character()
   } else {
     ncol1 <- df_ncol(out)
-    out <- mutate2(out, !!enquo(wt))
+    out_info <- mutate_summary_grouped(out, !!enquo(wt))
+    out <- out_info[["data"]]
     ncol2 <- df_ncol(out)
     has_wt <- (ncol2 == ncol1)
-    wt_var <- tidy_transform_names(data, !!enquo(wt))
+    wt_var <- out_info[["cols"]]
     if (length(wt_var) > 0L){
       wtv <- out[[wt_var]]
       if (!has_wt){
@@ -172,8 +176,10 @@ fadd_count <- function(data, ..., wt = NULL, sort = FALSE, order = TRUE,
       }
     }
   }
-  use_only_grouped_df_groups <- length(all_vars) == 0L ||
-    (order && length(group_vars) > 0L && length(group_vars) == length(all_vars))
+  use_only_grouped_df_groups <- !group_info[["groups_changed"]] && (
+    length(all_vars) == 0L ||
+      (order && length(group_vars) > 0L && length(group_vars) == length(all_vars))
+  )
   if (use_only_grouped_df_groups){
     g <- df_to_GRP(data, return.order = FALSE, order = order)
   } else {
