@@ -1,13 +1,7 @@
-#' Calculate time difference
+#' Time differences by any time unit
 #'
-#' @description This is a neat wrapper around lubridate's
-#' date/datetime arithmetic with support for numeric values and
-#' any class that supports arithmetic through "base::`-`"
-#' If either x or y are date/datetimes and time_type is "duration" or "period",
-#' then lubridate arithmetic is used, otherwise base R arithmetic is used.
-#'
-#' Some more exotic time units such as quarters, fortnights, etc
-#' can be specified.
+#' @description
+#' The time difference between 2 date or date-time vectors.
 #'
 #' @param x Start date or datetime.
 #' @param y End date or datetime.
@@ -25,6 +19,13 @@
 #' @returns
 #' A numeric vector recycled to the length of `max(length(x), length(y))`.
 #'
+#' @details
+#' When `time_by` is a numeric vector, e.g `time_by = 1` then
+#' base arithmetic using \code{base::`-`} is used, otherwise 'lubridate' style
+#' durations and periods are used.\cr
+#' Some more exotic time units such as quarters, fortnights, etcetera
+#' can be specified. See `.time_units` for more choices.
+#'
 #' @examples
 #' library(timeplyr)
 #' library(lubridate)
@@ -41,12 +42,20 @@
 #' time_diff(today(), today() + days(100),
 #'           time_by = list("days" = 1:100))
 #' time_diff(1, 1 + 0:100, time_by = 3)
+#' \donttest{
+#' library(nycflights13)
+#' library(bench)
+#'
+#' # Period differences are much faster
+#' mark(timeplyr = time_diff(flights$time_hour, today(), "weeks", time_type = "period"),
+#'      lubridate = interval(flights$time_hour, today()) / weeks(1))
+#' }
 #' \dontshow{
 #' data.table::setDTthreads(threads = .n_dt_threads)
 #' collapse::set_collapse(nthreads = .n_collapse_threads)
 #'}
 #' @export
-time_diff <- function(x, y, time_by = 1,
+time_diff <- function(x, y, time_by = 1L,
                       time_type = getOption("timeplyr.time_type", "auto")){
   tby <- time_by_list(time_by)
   units <- time_by_unit(tby)
@@ -59,8 +68,8 @@ time_diff <- function(x, y, time_by = 1,
     if (!is.numeric(x)){
       x <- time_as_number(x)
     }
-    out <- (y - x) / num
-    # out <- divide(y - x, num)
+    # out <- (y - x) / num
+    out <- divide(y - x, num)
   } else {
     time_type <- match_time_type(time_type)
     # Common but special case where from/to are whole days
@@ -72,10 +81,11 @@ time_diff <- function(x, y, time_by = 1,
                                                  time_type = time_type)
     if (is_special_case_days){
       if (units == "weeks"){
-        num <- num * 7
+        num <- num * 7L
       }
       by <- num
-      out <- (time_as_number(y) - time_as_number(x)) / by
+      out <- divide(time_as_number(y) - time_as_number(x), by)
+      # out <- (time_as_number(y) - time_as_number(x)) / by
       return(out)
     }
     if (time_type == "auto"){
@@ -84,12 +94,31 @@ time_diff <- function(x, y, time_by = 1,
     x <- as_datetime2(x)
     y <- as_datetime2(y)
     if (time_type == "period"){
+      # Use distinct start/end pairs (intervals)
+      # Instead of all of them because it's usually more efficient
+      interval_tbl <- list_to_data_frame(recycle_args(x = x, y = y, num = num))
+      interval_groups <- collapse::group(interval_tbl, starts = TRUE, group.sizes = TRUE)
+      starts <- attr(interval_groups, "starts")
+      sizes <- attr(interval_groups, "group.sizes")
+      n_groups <- attr(interval_groups, "N.groups")
+      # If distinct pairs results in a 2x reduction in data size, then we do that
+      distinct_pairs <- isTRUE((df_nrow(interval_tbl) %/% n_groups) >= 2L)
+      if ( distinct_pairs ){
+        interval_tbl <- df_row_slice(interval_tbl, starts)
+      }
+      x <- interval_tbl$x
+      y <- interval_tbl$y
+      num <- interval_tbl$num
+
       unit <- period_unit(units)(abs(num))
       out <- sign(num) * divide_interval_by_period2(x, y, unit)
       out[cpp_which(num == 0 & x > y)] <- -Inf
       out[cpp_which(num == 0 & x < y)] <- Inf
+      # Expand them back to original length
+      if (distinct_pairs){
+        out <- out[interval_groups]
+      }
     } else {
-      # unit <- duration_unit(units)(num)
       x <- time_as_number(x)
       y <- time_as_number(y)
       by <- unit_to_seconds(tby)
