@@ -3,7 +3,19 @@ data.table::setDTthreads(threads = 2L)
 # Set number of collapse threads to 1
 collapse::set_collapse(nthreads = 1L)
 
-testthat::test_that("Tests for time_countv", {
+time_countv2 <- function(..., include_interval = FALSE, use.names = TRUE, complete = TRUE){
+  out <- time_countv(..., include_interval = include_interval, complete = complete) %>%
+    dplyr::filter(!is.na(x))
+  if (!include_interval){
+   out <- fdeframe(out)
+   if (!use.names){
+     out <- unname(out)
+   }
+  }
+  out
+}
+
+testthat::test_that("Tests for time_countv2", {
   flights2 <- nycflights13::flights
   flights2 <- flights2 %>%
     dplyr::slice_sample(n = nrow(flights2)) %>%
@@ -11,14 +23,14 @@ testthat::test_that("Tests for time_countv", {
   from <- lubridate::as_datetime(lubridate::dmy(02042013)) +
     lubridate::minutes(35)
   to <- lubridate::dmy(08092013)
-  from2 <- bound_from(from, flights2$time_hour)
-  to2 <- bound_to(to, flights2$time_hour)
+  from2 <- time_cast(from, flights2$time_hour)
+  to2 <- time_cast(to, flights2$time_hour)
   nrow_flights2 <- flights2 %>%
     dplyr::filter(dplyr::between(time_hour, from2, to2)) %>%
     nrow()
   # Test for if the input order is retained, and whether the count is correct too
   res1 <- flights2 %>%
-    dplyr::mutate(n1 = time_countv(time_hour, time_by = "hour",
+    dplyr::mutate(n1 = time_countv2(time_hour, time_by = "hour",
                                    include_interval = FALSE, sort = FALSE, unique = FALSE, use.names = FALSE,
                                    complete = FALSE)) %>%
     fadd_count(time_hour, name = "n2") %>%
@@ -30,7 +42,7 @@ testthat::test_that("Tests for time_countv", {
   testthat::expect_identical(res1, dplyr::tibble(diff = 0L,
                                                  n = 6936L))
 
-  res2 <- time_countv(flights2$time_hour, use.names = FALSE)
+  res2 <- time_countv2(flights2$time_hour, use.names = FALSE)
 
   testthat::expect_equal(res2,
                              flights2 %>%
@@ -38,7 +50,7 @@ testthat::test_that("Tests for time_countv", {
                                time_complete(time = time_hour,
                                              fill = list(n = 0)) %>%
                                dplyr::pull(n))
-  res3 <- time_countv(flights2$time_hour, use.names = FALSE,
+  res3 <- time_countv2(flights2$time_hour, use.names = FALSE,
                       from = from, to = to)
   testthat::expect_equal(res2,
                              flights2 %>%
@@ -46,7 +58,7 @@ testthat::test_that("Tests for time_countv", {
                                time_complete(time = time_hour,
                                              fill = list(n = 0)) %>%
                                dplyr::pull(n))
-  res4 <- time_countv(flights2$time_hour,
+  res4 <- time_countv2(flights2$time_hour,
                       time_by = "month",
                       from = from, to = to)
   testthat::expect_equal(table(time_aggregate_left(flights2$time_hour,
@@ -75,7 +87,7 @@ testthat::test_that("Tests for time_countv", {
                            dplyr::pull(n) %>%
                            add_names(tbreaks))
 
-  res5 <- time_countv(flights2$time_hour, time_by = "month",
+  res5 <- time_countv2(flights2$time_hour, time_by = "month",
                       from = from, to = to, sort = FALSE,
                       use.names = FALSE)
 
@@ -90,19 +102,19 @@ testthat::test_that("Tests for time_countv", {
                            dplyr::pull(n))
 
   # Unfinished
-  res6 <- time_countv(flights2$time_hour, time_by = "month",
+  res6 <- time_countv2(flights2$time_hour, time_by = "month",
                       from = from, to = to, sort = FALSE,
                       use.names = TRUE,
                       include_interval = TRUE)
-  res7 <- time_countv(flights2$time_hour, time_by = "month",
+  res7 <- time_countv2(flights2$time_hour, time_by = "month",
                       from = from, to = to, sort = FALSE, unique = FALSE,
                       use.names = TRUE,
                       include_interval = TRUE)
-  res8 <- time_countv(flights2$time_hour, time_by = "month",
+  res8 <- time_countv2(flights2$time_hour, time_by = "month",
                       from = from, to = to, sort = TRUE, unique = FALSE,
                       use.names = TRUE,
                       include_interval = TRUE)
-  res9 <- time_countv(flights2$time_hour, time_by = "month",
+  res9 <- time_countv2(flights2$time_hour, time_by = "month",
                       from = from, to = to, sort = TRUE, unique = TRUE,
                       use.names = FALSE,
                       include_interval = TRUE)
@@ -116,7 +128,7 @@ testthat::test_that("Tests for time_countv", {
                            fcount(x) %>%
                            dplyr::mutate(interval = tseq_interval(to, x)) %>%
                            dplyr::select(x, interval, n))
-  res9a <- time_countv(flights2$time_hour,
+  res9a <- time_countv2(flights2$time_hour,
                        time_by = "hour", include_interval = TRUE)
   res9b <- flights2 %>%
     fcount(x = time_hour) %>%
@@ -128,33 +140,33 @@ testthat::test_that("Tests for time_countv", {
   testthat::expect_true(nrow(dplyr::anti_join(res9a, res9b)) == 0L)
   testthat::expect_true(nrow(dplyr::anti_join(res9b, res9a)) == 0L)
   testthat::expect_identical(
-    time_countv(flights2$time_hour, time_by = "3.5 hours", include_interval = TRUE) %>%
+    time_countv2(flights2$time_hour, time_by = "3.5 hours", include_interval = TRUE) %>%
       dplyr::filter(interval / duration_unit("hours")(1) > 3.5) %>%
       nrow(),
     0L
   )
   # Unfinished
-  res10 <- time_countv(flights2$time_hour, time_by = "month",
+  res10 <- time_countv2(flights2$time_hour, time_by = "month",
                        from = from, to = to, sort = FALSE,
                        use.names = TRUE,
                        include_interval = FALSE)
-  res11 <- time_countv(flights2$time_hour, time_by = "month",
+  res11 <- time_countv2(flights2$time_hour, time_by = "month",
                        from = from, to = to, sort = FALSE, unique = FALSE,
                        use.names = TRUE,
                        include_interval = FALSE)
-  res12 <- time_countv(flights2$time_hour, time_by = "month",
+  res12 <- time_countv2(flights2$time_hour, time_by = "month",
                        from = from, to = to, sort = TRUE, unique = FALSE,
                        use.names = TRUE,
                        include_interval = FALSE)
-  res13 <- time_countv(flights2$time_hour, time_by = "month",
+  res13 <- time_countv2(flights2$time_hour, time_by = "month",
                        from = from, to = to, sort = TRUE, unique = TRUE,
                        use.names = TRUE,
                        include_interval = FALSE)
   x <- flights2$time_hour
-  res <- time_countv(x, sort = FALSE, unique = TRUE, time_by = "2 weeks", include_interval = TRUE)
+  res <- time_countv2(x, sort = FALSE, unique = TRUE, time_by = "2 weeks", include_interval = TRUE)
   testthat::expect_equal(res$x,
                          lubridate::int_start(res$interval))
-  res <- time_countv(x[c(1:100, 30000:35000, 100000:100001)],
+  res <- time_countv2(x[c(1:100, 30000:35000, 100000:100001)],
                          sort = TRUE, unique = FALSE, time_by = "week", include_interval = TRUE,
                          time_floor = TRUE)
   testthat::expect_equal(res$x,
