@@ -19,7 +19,7 @@ testthat::test_that("time breaks", {
   res2 <- time_breaks(x, n = 5, time_by = "week")
   res3 <- time_breaks(x, n = 100, time_by = "month")
   res4 <- time_breaks(x, n = 100, time_by = "month", time_type = "duration")
-  res5 <- time_breaks(x, n = 5, time_by = "week", n_at_most = FALSE)
+  # res5 <- time_breaks(x, n = 5, time_by = "week", n_at_most = FALSE)
   testthat::expect_equal(res3, time_span(x, time_by = "month"))
   testthat::expect_equal(res4, time_span(x, time_by = "month",
                                              time_type = "duration"))
@@ -31,11 +31,11 @@ testthat::test_that("time breaks", {
                                    dplyr::lag(res2),
                                    time_by = "weeks", time_type = "period"),
                          c(NA, rep(-11, 4)))
-  testthat::expect_equal(time_diff(res5,
-                                   dplyr::lag(res5),
-                                   time_by = "weeks", time_type = "period"),
-                         c(NA, rep(-10, 5)))
-  testthat::expect_error(supressWarnings(time_breaks(x, n = Inf)))
+  # testthat::expect_equal(time_diff(res5,
+  #                                  dplyr::lag(res5),
+  #                                  time_by = "weeks", time_type = "period"),
+  #                        c(NA, rep(-10, 5)))
+  # testthat::expect_error(supressWarnings(time_breaks(x, n = Inf)))
   testthat::expect_equal(time_breaks(x, n = 100, time_by = "month",
                                          from = start1,
                                          to = end2 + period_unit("months")(4)),
@@ -50,8 +50,7 @@ testthat::test_that("time breaks", {
                                          to = end2),
                              time_span(x, time_by = "month",
                                        to = end2))
-  testthat::expect_equal(time_breaks(x, n = 10^6, time_by = "hour",
-                                     n_at_most = FALSE),
+  testthat::expect_equal(time_breaks(x, n = Inf, time_by = "hour"),
                              time_span(x, time_by = "hour"))
 })
 
